;
; MIOS Example Driver for SRAM Module
;
; 128k x 8 and 512k x 8 SRAM type are supported
; 
; the page read/write routines DON'T check the address range 
; for speed reasons
;
; You may need to modify the Variables section,
; depending on your application
;
; ==========================================================================
;
; Copyright (C) 2005 by TL (tl@automatic-brain.de)
;
; ==========================================================================
;
; This file is part of SRAM_Test1
;
; SRAM_Test1 is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; <application> is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with <application>; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================


;; --------------------------------------------------------------------------
;;  User-Variables
;; --------------------------------------------------------------------------

; Note: SRAM_ADDR has to be defined in app_defines.h
; SRAM_ADDR		EQU	0x07d	; A16 - A18
; SRAM_ADDR+1				; A8  - A15
; SRAM_ADDR+2				; A0  - A7

; the address in PIC data memory where to map a page (64bytes) from SRAM
; must be 64 bytes aligned
#define SRAM_MAP 0x100

;; --------------------------------------------------------------------------
;;  Constants for CS/OE pins of SRAM and LW of latches
;; --------------------------------------------------------------------------
#define SRAM_PORT_LATCH1_LE     PORTA
#define SRAM_LAT_LATCH1_LE      LATA
#define SRAM_TRIS_LATCH1_LE     TRISA
#define SRAM_PIN_LATCH1_LE      0

#define SRAM_PORT_LATCH2_LE     PORTA
#define SRAM_LAT_LATCH2_LE      LATA
#define SRAM_TRIS_LATCH2_LE     TRISA
#define SRAM_PIN_LATCH2_LE      1

#define SRAM_PORT_CS            PORTA
#define SRAM_LAT_CS             LATA
#define SRAM_TRIS_CS            TRISA
#define SRAM_PIN_CS             2

#define SRAM_PORT_WE            PORTA
#define SRAM_LAT_WE             LATA
#define SRAM_TRIS_WE            TRISA
#define SRAM_PIN_WE             3

#define SRAM_PORT_OE            PORTA
#define SRAM_LAT_OE             LATA
#define SRAM_TRIS_OE            TRISA
#define SRAM_PIN_OE             5

;; --------------------------------------------------------------------------
;;  This function is called after startup to initialize the SRAM
;; --------------------------------------------------------------------------
SRAM_Init
	;; disable the ADC which allocates the analog pins
	;; only needed if controls pins are connected to port A
	movlw	0x07
	movwf	ADCON1

	; put PortA in output mode
	bcf		SRAM_TRIS_LATCH1_LE, SRAM_PIN_LATCH1_LE
	bcf		SRAM_TRIS_LATCH2_LE, SRAM_PIN_LATCH2_LE
	bcf		SRAM_TRIS_CS, SRAM_PIN_CS
	bcf		SRAM_TRIS_WE, SRAM_PIN_WE
	bcf		SRAM_TRIS_OE, SRAM_PIN_OE
	; set control lines
	bsf		SRAM_PORT_OE, SRAM_PIN_OE
	bsf		SRAM_PORT_WE, SRAM_PIN_WE
	bsf		SRAM_PORT_CS, SRAM_PIN_CS
	bcf		SRAM_PORT_LATCH1_LE, SRAM_PIN_LATCH1_LE
	bcf		SRAM_PORT_LATCH2_LE, SRAM_PIN_LATCH2_LE

	; put PortE in output mode
	bcf		TRISE, 0
	bcf		TRISE, 1
	bcf		TRISE, 2


	return

;; --------------------------------------------------------------------------
;;  This function is called to read a byte from the SRAM
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR   = A16-A18
;;      SRAM_ADDR+1 = A8-A15
;;      SRAM_ADDR+2 = A0-A7
;;  Output:
;;    o byte read from SRAM in MIOS_PARAMETER1 and WREG
;; --------------------------------------------------------------------------
SRAM_Read
	; first latch
	movff 	SRAM_ADDR+2, PORTB
	bsf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE
	bcf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE

	; second latch
	movff 	SRAM_ADDR+1,PORTB
	bsf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE
	bcf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE

	; remaining address lines
	movff	SRAM_ADDR, PORTE

	; enable port read mode
	setf	TRISB

	bcf 	SRAM_LAT_CS, SRAM_PIN_CS
	bcf 	SRAM_LAT_OE, SRAM_PIN_OE

	nop

	; read data
	movff 	PORTB, MIOS_PARAMETER1

	bsf 	SRAM_LAT_OE, SRAM_PIN_OE
	bsf 	SRAM_LAT_CS, SRAM_PIN_CS

	; disable port read mode
	clrf	TRISB
	movf    MIOS_PARAMETER1, W
	return


;; --------------------------------------------------------------------------
;;  This function is called to write a byte to the SRAM
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR   = A18-A16
;;      SRAM_ADDR+1 = A8-A15 
;;      SRAM_ADDR+2 = A0-A7
;;    o byte to write in MIOS_PARAMETER1
;; --------------------------------------------------------------------------
SRAM_Write

	; first latch
	movff 	SRAM_ADDR+2, PORTB
	bsf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE
	bcf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH1_LE

	; second latch
	movff	SRAM_ADDR+1, PORTB
	bsf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE
	bcf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE

	; remaining address lines
	movff 	SRAM_ADDR, PORTE

	; write byte
	movff 	MIOS_PARAMETER1, PORTB

	bcf 	SRAM_LAT_CS, SRAM_PIN_CS
	bcf 	SRAM_LAT_WE, SRAM_PIN_WE

	nop

	bsf 	SRAM_LAT_WE, SRAM_PIN_WE
	bsf 	SRAM_LAT_CS, SRAM_PIN_CS

	return


;; --------------------------------------------------------------------------
;;  This function is called to read a page (64bytes) from the SRAM
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR       = A16-A18
;;      SRAM_ADDR+1     = A8-A15
;;      SRAM_ADDR+2     = A0-A7
;;
;;     Notes:
;;      NO RANGE CHECKING is performed
;;      SRAM_ADDR has to be 64 bytes aligned
;;  Output:
;;    o in mapped pic data memory the 64 bytes read from SRAM
;;
;;  Uses:
;;    o FSR1
;; --------------------------------------------------------------------------
SRAM_Read_Page
	; second latch
	movff 	SRAM_ADDR+1,PORTB
	bsf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE
	bcf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE

	; remaining address lines
	movff	SRAM_ADDR, PORTE

        lfsr    FSR1, SRAM_MAP
        ; make sure SRAM_MAP is 64 byte aligned
        movlw   0xc0
        andwf   FSR1L, F
        ; make sure SRAM_ADDR is 64 byte aligned
        movff   SRAM_ADDR+2, TMP1
        movlw   0xc0
        andwf   TMP1, F
        movf    TMP1, W
        addlw   0x40
SRAM_Read_Page_Loop
        ; first latch
        movff 	TMP1, PORTB
	bsf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE
	bcf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE

	; enable port read mode
	setf	TRISB
	bcf 	SRAM_LAT_CS, SRAM_PIN_CS
	bcf 	SRAM_LAT_OE, SRAM_PIN_OE
	nop
	; read data into PIC memory
	movff 	PORTB, POSTINC1
	bsf 	SRAM_LAT_OE, SRAM_PIN_OE
	bsf 	SRAM_LAT_CS, SRAM_PIN_CS
	; disable port read mode
	clrf	TRISB
	incf    TMP1, F
	cpfseq  TMP1
	bra     SRAM_Read_Page_Loop
	return

;; --------------------------------------------------------------------------
;;  This function is called to write a page (64 bytes) to the SRAM from
;;  PIC internal data memory
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR       = A16-A18
;;      SRAM_ADDR+1     = A8-A15
;;      SRAM_ADDR+2     = A0-A7
;;
;;     Notes:
;;      NO RANGE CHECKING is performed
;;      SRAM_ADDR has to be 64 bytes aligned
;;
;;  Uses:
;;    o FSR1
;; --------------------------------------------------------------------------
SRAM_Write_Page
	; second latch
	movff 	SRAM_ADDR+1,PORTB
	bsf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE
	bcf 	SRAM_LAT_LATCH2_LE, SRAM_PIN_LATCH2_LE

	; remaining address lines
	movff	SRAM_ADDR, PORTE

        lfsr    FSR1, SRAM_MAP
        ; make sure SRAM_MAP is 64 byte aligned
        movlw   0xc0
        andwf   FSR1L, F
        ; make sure SRAM_ADDR is 64 byte aligned
        movff   SRAM_ADDR+2, TMP1
        movlw   0xc0
        andwf   TMP1, F
        movf    TMP1, W
        addlw   0x40
SRAM_Write_Page_Loop
        ; first latch
        movff 	TMP1, PORTB
	bsf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE
	bcf 	SRAM_LAT_LATCH1_LE, SRAM_PIN_LATCH1_LE
	; write byte
	movff 	POSTINC1, PORTB
	bcf 	SRAM_LAT_CS, SRAM_PIN_CS
	bcf 	SRAM_LAT_WE, SRAM_PIN_WE
	nop
	bsf 	SRAM_LAT_WE, SRAM_PIN_WE
	bsf 	SRAM_LAT_CS, SRAM_PIN_CS
	incf    TMP1, F
	cpfseq  TMP1
	bra     SRAM_Write_Page_Loop
	return


